
{*******************************************************}
{                                                       }
{       TWavButton                                      }
{       Button playing WAV file on click                }
{                                                       }
{       Copyright (c) Sergey Tkachenko                  }
{       svt@trichview.com                               }
{       http://www.trichview.com                        }
{                                                       }
{*******************************************************}

{
  TWavButton component (appears on the tab "TRichView Misc")

  The main feature of this component: storing WAV data inside.
  When inserted in TRichView, it allows storing WAV in RVF file.
  Main methods:
    LoadFromFile - loads WAV file
    Clear - clears stored WAV data
  Property:
    FileName. Assigning to this property calls LoadFromFile. Value of this
    property is not stored. Using this property, you can load file at design
    time using the Object Inspector.
}


unit WavButton;

interface

uses Windows, Classes, SysUtils, StdCtrls, MMSystem;

type
  TWavButton = class(TButton)
  private
    { Private declarations }
    FFileName: TFileName;
    FStream: TMemoryStream;
    procedure WriteMusic(Stream: TStream);
    procedure ReadMusic(Stream: TStream);
    procedure SetFileName(const Value: TFileName);
  protected
    { Protected declarations }
    procedure DefineProperties(Filer: TFiler); override;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function LoadFromFile(const FileName: String): Boolean;
    procedure Clear;
    procedure Click; override;
  published
    { Published declarations }
    property FileName: TFileName read FFileName write SetFileName stored False;
  end;

procedure Register;  

implementation

{ TWavButton }

constructor TWavButton.Create(AOwner: TComponent);
begin
  inherited;
  FStream := TMemoryStream.Create;
end;

destructor TWavButton.Destroy;
begin
  FStream.Free;
  inherited;
end;

procedure TWavButton.Clear;
begin
  FStream.Clear
end;

function TWavButton.LoadFromFile(const FileName: String): Boolean;
var fs: TFileStream;
begin
  Result := False;
  try
    fs := TFileStream.Create(FileName, fmOpenRead);
    FStream.SetSize(0);
    FStream.CopyFrom(fs, fs.Size);
    fs.Free;
    Result := True;
  except;
  end;
end;

procedure TWavButton.DefineProperties(Filer: TFiler);
begin
  inherited;
  Filer.DefineBinaryProperty('Music', ReadMusic, WriteMusic, FStream.Size>0);
end;


procedure TWavButton.ReadMusic(Stream: TStream);
var v: Integer;
begin
  Stream.ReadBuffer(v, sizeof(v));
  if v>0 then begin
    FStream.SetSize(0);
    FStream.CopyFrom(Stream, v);
  end;
end;

procedure TWavButton.WriteMusic(Stream: TStream);
var v: Integer;
begin
  v := FStream.Size;
  Stream.WriteBuffer(v, sizeof(v));
  FStream.Position := 0;
  Stream.CopyFrom(FStream, v)
end;

procedure TWavButton.Click;
begin
  inherited;
  if FStream.Size=0 then
    exit;
  Enabled := False;
  try
    PlaySound(FStream.Memory, 0, SND_SYNC or SND_MEMORY);
  finally
    Enabled := True;
  end;
end;

procedure TWavButton.SetFileName(const Value: TFileName);
begin
  if Value='' then begin
    FFileName := '';
    Clear
    end
  else begin
    if Value[1]='<' then
      exit;
    if LoadFromFile(Value) then
      FFileName := '<Loaded: '+ExtractFileName(Value)+'>'
    else
      FFileName := '<Failed: '+ExtractFileName(Value)+'>'
  end;
end;

procedure Register;
begin
  RegisterComponents('RichView Misc', [TWavButton]);
end;


end.
